import { scaffolderPlugin } from '@backstage/plugin-scaffolder';
import { createScaffolderFieldExtension } from '@backstage/plugin-scaffolder-react';
import React, { useState } from 'react';
import FormControl from '@material-ui/core/FormControl';
import { useApi, discoveryApiRef, fetchApiRef } from '@backstage/core-plugin-api';
import { ErrorPanel, Progress, Select } from '@backstage/core-components';
import { useAsync } from 'react-use';
import get from 'lodash/get';
import sortBy from 'lodash/sortBy';
import { z } from 'zod';
import { FormHelperText } from '@material-ui/core';

const selectFieldFromApiConfigSchema = z.object({
  params: z.record(z.string(), z.string()).optional(),
  path: z.string(),
  arraySelector: z.string().or(z.array(z.string())).optional(),
  valueSelector: z.string().or(z.array(z.string())).optional(),
  labelSelector: z.string().or(z.array(z.string())).optional(),
  title: z.string().optional(),
  description: z.string().optional()
});

const SelectFieldFromApi = (props) => {
  var _a;
  const discoveryApi = useApi(discoveryApiRef);
  const fetchApi = useApi(fetchApiRef);
  const [dropDownData, setDropDownData] = useState();
  const options = selectFieldFromApiConfigSchema.parse(
    props.uiSchema["ui:options"]
  );
  const { title = "Select", description = "" } = options;
  const { error } = useAsync(async () => {
    const baseUrl = await discoveryApi.getBaseUrl("");
    const params = new URLSearchParams(options.params);
    const response = await fetchApi.fetch(
      `${baseUrl}${options.path}?${params}`
    );
    const body = await response.json();
    const array = options.arraySelector ? get(body, options.arraySelector) : body;
    const constructedData = array.map((item) => {
      let value;
      let label;
      if (options.valueSelector) {
        value = get(item, options.valueSelector);
        label = options.labelSelector ? get(item, options.labelSelector) : value;
      } else {
        if (!(typeof item === "string")) {
          throw new Error(
            `The item provided for the select drop down "${item}" is not a string`
          );
        }
        value = item;
        label = item;
      }
      if (!value) {
        throw new Error(`Failed to populate SelectFieldFromApi dropdown`);
      }
      return {
        value,
        label: label || value
      };
    });
    setDropDownData(sortBy(constructedData, "label"));
  });
  if (error) {
    return /* @__PURE__ */ React.createElement(ErrorPanel, { error });
  }
  if (!dropDownData) {
    return /* @__PURE__ */ React.createElement(Progress, null);
  }
  return /* @__PURE__ */ React.createElement(
    FormControl,
    {
      margin: "normal",
      required: props.required,
      error: ((_a = props.rawErrors) == null ? void 0 : _a.length) > 0 && !props.formData
    },
    /* @__PURE__ */ React.createElement(
      Select,
      {
        items: dropDownData,
        label: title,
        onChange: props.onChange,
        native: true
      }
    ),
    /* @__PURE__ */ React.createElement(FormHelperText, null, description)
  );
};

const SelectFieldFromApiExtension = scaffolderPlugin.provide(
  createScaffolderFieldExtension({
    name: "SelectFieldFromApi",
    component: SelectFieldFromApi
  })
);

export { SelectFieldFromApiExtension };
//# sourceMappingURL=index.esm.js.map
